#!/usr/bin/env python

from qt import *
from oggripper import MainWindow
from folderitem import FolderItem

import os, sys, re, CDDB, DiscID

class MainWindowImpl(MainWindow):
    def __init__(self, *args):
        apply(MainWindow.__init__,(self,) + args)
    
    def eject(self, *args):
        self.listTracks.clear()
        self.labelHelp.setText("Ejecting...")
        os.system("eject")
        self.labelHelp.setText("Ready...")
        
    def stop(self, *args):
        
        if self.pcdp.isRunning() or self.penc.isRunning():
            
            self.tracks = []
            
            if self.pcdp.isRunning(): self.pcdp.kill()
            if self.penc.isRunning(): self.penc.kill()
            
            self.buttonQuery.setEnabled(True)
            self.buttonEject.setEnabled(True)
            self.buttonOptions.setEnabled(True)
            self.buttonRipTracks.setEnabled(True)
            
        else:
            qApp.exit()
    
    def folderClicked(self, item):
        if not item: return
        
        self.lineFolder.setText(item.fileName())
    
    def init(self, *args):
        
        h = FolderItem(self.listFolders, unicode(QDir.home().path())+'/', True)
        h.setOpen(True)
        r = FolderItem(self.listFolders, '/', True)
        r.setOpen(False)
        
        self.listTracks.setColumnWidth(0,50)
        self.listTracks.setColumnWidth(2,70)
        self.listTracks.setColumnWidthMode(1, QListView.Maximum)
        
        self.tab.setCurrentPage(1)
        
        self.comboboxDevice.insertItem("/dev/cdrom")
        
        self.tracks = []
        self.pcdp = QProcess()
        self.penc = QProcess()
        
        self.re_from = re.compile('from\s+sector\s+(\d+)')
        self.re_to   = re.compile('to\s+sector\s+(\d+)')
        self.re_read = re.compile('read.\s.\s(\d+)')
        
    def options(self, *args):
        print "MainWindowImpl.options(): Not implemented yet"
    
    def query(self):
        
        self.tab.setCurrentPage(0)
        self.listTracks.clear()
        
        self.listTracks.setColumnWidth(0,50)
        self.listTracks.setColumnWidth(1,self.listTracks.width()-120-5)
        self.listTracks.setColumnWidth(2,70)

        self.lineTitle.setText("")
        self.lineArtist.setText("")
        
        self.labelHelp.setText("Querying CD for media info...")
        
        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
        qApp.processEvents()
        
        cdrom = DiscID.open()
        disc_id = DiscID.disc_id(cdrom)
        
        num_tracks = disc_id[1]
        
        for i in range(num_tracks):
            num = " %.02d." % (i+1)
            track = QListViewItem(self.listTracks, num, "Track")
            track.setPixmap(0, QPixmap("pics/track.png"));
        
        self.labelHelp.setText("Receiving info. Wait...")
        qApp.processEvents()
        
        try:
            (query_status, query_info) = CDDB.query(disc_id)
            (read_status, read_info) = CDDB.read(query_info['category'], query_info['disc_id'])
            
            self.listTracks.clear()
            for i in range(num_tracks):
                num = " %.02d." % (i+1)
                track = QListViewItem(self.listTracks, num, read_info['TTITLE' + `i`])
                track.setPixmap(0, QPixmap("pics/track.png"));
            
            dtitle = read_info['DTITLE']
            try:
                artist, title = dtitle.split('/')
            except:
                artist, title = '', dtitle
            
            self.lineTitle.setText(title.strip())
            self.lineArtist.setText(artist.strip())
            
        except:
            self.lineTitle.setText('Unknow')
            self.lineArtist.setText('Somebody')
            pass
        
        QApplication.restoreOverrideCursor()
        self.labelHelp.setText("Ready...")
    
    def selectFolder(self, *args):
        print "MainWindowImpl.selectFolder(): Not implemented yet"
    
    def ripTracks(self, *args):
        self.tracks = []
        
        li = self.listTracks.firstChild()
        
        if not li:
            QMessageBox.information(self, \
                "tkcOggRipper.listTracks", \
                "I don't see any track.\nTry to query CD before ripping.", \
                "Ok")
            return
        
        while li:
            if li.isSelected():
                num = int(str(li.text(0)).strip(' .'))
                self.tracks.append(num)
            
            li = li.nextSibling()
        
        if len(self.tracks):
            self.tracksCount = len(self.tracks)
            self.tracksRipped = 0
            self.ripTrack(self.tracks[0])
        
        else: 
            # we will rip all tracks
            li = self.listTracks.firstChild()
            while (li):
                self.listTracks.setSelected(li, True)
                li = li.nextSibling()
            
            self.ripTracks()
    
    def ripTrack(self, track):
        
        self.progressTrack.setProgress(0)
        
        if self.pcdp.isRunning(): self.pcdp.kill()
        if self.penc.isRunning(): self.penc.kill()
        
        self.pcdp.clearArguments()
        
        self.pcdp.addArgument("cdparanoia")
        self.pcdp.addArgument("-e")
        self.pcdp.addArgument("-r")
        self.pcdp.addArgument("--")
        self.pcdp.addArgument(str(track))
        self.pcdp.addArgument("-")
        
        self.penc.clearArguments()
        
        self.penc.addArgument("oggenc")
        self.penc.addArgument("--quality=5")
        self.penc.addArgument("-r")
        self.penc.addArgument("-o")
        
        title = unicode(self.lineTitle.text()).strip()
        artist = unicode(self.lineArtist.text()).strip()
        fdir = "%s - %s" % ( artist, title )
        fdir = re.sub("[\\\/\.\,\:\;]", "", fdir)
        
        folder = unicode(self.lineFolder.text()).strip()
        
        if folder == '':
            folder = unicode(QDir.homeDirPath())
            
        if folder[-1] != '/': folder += '/'
        self.lineFolder.setText(folder)
        
        self.createDir(folder+fdir)
        folder = folder + fdir
        if folder[-1] != '/': folder += '/'
        
        trackName = "Track"
        li = self.listTracks.firstChild()
        while li:
            num = int(str(li.text(0)).strip(' .'))
            if track == num: trackName = unicode(li.text(1))
            li = li.nextSibling()
        
        trackName = re.sub("[\\\/\.\,\:\;]", "", trackName)
        
        file = "%d - %s" % (track, trackName)
        fullName = ""+folder+file+".ogg"
        
        self.penc.addArgument(fullName)
        
        self.penc.addArgument("-N")
        self.penc.addArgument(str(track))
        
        self.penc.addArgument("-l")
        self.penc.addArgument(self.lineTitle.text())
        
        self.penc.addArgument("-a")
        self.penc.addArgument(self.lineArtist.text())
        
        self.penc.addArgument("-t")
        self.penc.addArgument(trackName)
        self.penc.addArgument("-")
        
        self.connect(self.penc, SIGNAL('processExited()'), self.encFinished)
        
        self.connect(self.pcdp, SIGNAL('readyReadStdout()'), self.ripStdout)
        self.connect(self.pcdp, SIGNAL('readyReadStderr()'), self.ripStderr)
        self.connect(self.pcdp, SIGNAL('processExited()'), self.ripFinished)
        
        self.fromSector = 0
        self.toSector = 0
        
        if not self.pcdp.start():
            QMessageBox.information(self, \
                "tkcOggRipper.ripTrack", \
                "I can't run 'cdparanoia' program.", \
                "Ok")
            
            self.labelHelp.setText("Can't run cdparanoia command...")
            return
       
        elif not self.penc.start():
            self.pcdp.kill()
            QMessageBox.information(self, \
                "tkcOggRipper::ripTracks", \
                "I can't run 'oggenc' program.", \
                "Ok")
            
            self.labelHelp.setText("Can't run oggenc command...")
            return
        
        else: self.labelHelp.setText("Ripping/Encoding...")
        
        self.buttonQuery.setEnabled(False)
        self.buttonEject.setEnabled(False)
        self.buttonOptions.setEnabled(False)
        self.buttonRipTracks.setEnabled(False)
    
    def ripFinished(self, *args):
   
        self.disconnect(self.pcdp, SIGNAL('readyReadStdout()'), self.ripStdout)
        self.disconnect(self.pcdp, SIGNAL('readyReadStderr()'), self.ripStderr)
        self.disconnect(self.pcdp, SIGNAL('processExited()'), self.ripFinished)
        
        self.penc.closeStdin()
    
    def ripStderr(self, *args):
        
        while self.pcdp.canReadLineStderr():
            
            text = unicode(self.pcdp.readLineStderr())
            
            m = self.re_from.search(text)
            if m:
                self.fromSector = int(m.group(1))
            
            m = self.re_to.search(text)
            if m:
                self.toSector = int(m.group(1))
            
            m = self.re_read.search(text)   
            if m:
                sector = int(m.group(1))/1176
                
                progress = ((sector-self.fromSector)*100) / (self.toSector-self.fromSector)
                
                if progress > self.progressTrack.progress():
                    
                    self.progressTrack.setProgress(progress)
                    
                    disk_progress = (self.tracksRipped*100+progress)/self.tracksCount
                    
                    self.progressDisc.setProgress(disk_progress)
                    self.setCaption("Rip - "+str(disk_progress)+"%")
    
    def ripStdout(self, *args):
        self.penc.writeToStdin(self.pcdp.readStdout())
    
    def unselectTrack(self, track):
        
        li = self.listTracks.firstChild()
        while li:
            num = int(str(li.text(0)).strip(' .'))
            
            if track == num:
                self.listTracks.setSelected(li, False)
            
            li = li.nextSibling()
    
    def encFinished(self, *args):
        
        self.labelHelp.setText("Ready")
        self.disconnect(self.penc, SIGNAL('processExited()'), self.encFinished)
        
        if len(self.tracks):
            self.unselectTrack(self.tracks[0])
            self.tracks = self.tracks[1:]
        
        else:
            # user press exit
            self.setCaption("tkcOggRipper")
            self.progressDisc.setProgress(0)
            self.progressTrack.setProgress(0)
            return
            
        self.tracksRipped += 1
        self.progressTrack.setProgress(0)
        self.progressDisc.setProgress((self.tracksRipped * 100)/self.tracksCount)
        
        self.setCaption("Rip - "+str((self.tracksRipped*100)/self.tracksCount)+"%")
    
        if len(self.tracks):
            
            self.ripTrack(self.tracks[0])
        
        else:
            self.setCaption("tkcOggRipper")
            self.progressDisc.setProgress(0)
            
            self.buttonQuery.setEnabled(True)
            self.buttonEject.setEnabled(True)
            self.buttonOptions.setEnabled(True)
            self.buttonRipTracks.setEnabled(True)
    
    def createDir(self, fdir, dcount=0):
        
        if dcount>10: return False
        
        if fdir[-1]=='/': fdir = fdir[:-1]
        
        d = QDir(fdir)
        if not d.exists():
            try:
                items = fdir.split('/')
            except:
                return False
            
            items = items[:-1]
            
            left = '/'.join(items)
            
            dcount += 1
            
            if self.createDir(left, dcount):
                d.mkdir(fdir)
            else: 
                return False
            
        return True
    